/*
**
** FileInfosC.cpp
**
** $VER: FileInfoC.cpp 1.1 (13.04.98)
**
** $Revision: 1.3 $
** $State: Exp $
** $Date: 1998/08/16 19:03:36 $
**
** $Log: FileInfosC.cpp $
** Revision 1.3  1998/08/16 19:03:36  kakace
** Version Beta3+
**
** Revision 1.2  1998/08/02 15:17:03  kakace
** Fonctionnement OK (Layer 1/2 + Stereo/JStereo)
**
** Revision 1.1  1998/07/24 14:26:52  kakace
** Automatic global CheckIn
**
** Revision 1.19  1998/07/24 14:14:25  kakace
** Version intermdiaire (CheckIn global)
**
** Revision 1.17  1998/07/02 15:51:59  kakace
** Automatic global CheckIn
**
*/


//----------------------------------------------------------------------------------------------------

/// Includes

#ifndef  _INCLUDE_STDLIB_H
#include <stdlib.h>
#endif

#ifndef  _PEGASE_HPP
#include "Pegase.hpp"
#endif

#ifndef  _FILEINFOS_CLASS_HPP
#include "FileInfosC.hpp"
#endif

#ifndef  CLASS_DISPLAY_HPP
#include "Display.hpp"
#endif

#define  CATCOMP_NUMBERS
#ifndef  PEGASE_H
#include "Pegase.h"
#endif

///


//----------------------------------------------------------------------------------------------------
//======================================== Fonctions locales =========================================
//----------------------------------------------------------------------------------------------------

// Allouer un tampon et y copier la chane transmise en argument.

inline char *DuplicateString(STRING str)
{
    char *s = NULL;

    if (str && *str)
    {
        s = new char[ strlen(str)+1 ];

        if (s) strcpy(s, str);
    }

    return s;                               // Chane nulle ou pointeur nul.
}


//----------------------------------------------------------------------------------------------------
//========================================= class FileInfos ==========================================
//----------------------------------------------------------------------------------------------------

/// FileInfosC::CONSTRUCTOR
/****** Class FileInfosC/CONSTRUCTOR ****************************************
*
*   NAME
*   FileInfosC::FileInfosC -- Constructeur
*
*   SYNOPSIS
*   FileInfosC::FileInfosC(cFileName, cPath, roConfig)
*
*   FileInfosC::FileInfosC(STRING, STRING, EncoderConfigC);
*
*   FUNCTION
*   Cre un objet FileInfos pour le fichier indiqu.
*   Le nom du fichier et le nom du rpertoire parent sont recopis dans un
*   tampon priv.
*
*   INPUTS
*   cFileName - Nom du fichier  examiner.
*   cPath     - Nom du rpertoire contenant ce fichier.
*   roConfig  - Paramtres choisis pour l'encodeur.
*
*   SEE ALSO
*   DESTRUCTOR.
*
*****************************************************************************
*
*/


FileInfosC::FileInfosC(STRING cFileName, STRING cPath, EncoderConfigC &roConfig) : EncoderConfigC(roConfig), fi_roDefConfig(roConfig)
{
    char *name;

    // Dupliquer le nom du fichier ainsi que son chemin d'accs complet.

    name         = DuplicateString(cFileName);
    fi_cFullPath = DuplicateString(cPath);

    // Initialiser les membres donnes.

    ComputeTime();

    if (name && fi_cFullPath)
    {
        NodeName(name);                          // Le nom du fichier constitue le nom du noeud.
        NodeType(AFT_UNKNOWN);                   // Type de fichier inconnu (pour le moment).
    }
    else
    {
        fi_iIoError = IOERR_NO_MEMORY;
    }
}
///
/// FileInfosC::DESTRUCTOR
/****** Class FileInfosC/DESTRUCTOR *****************************************
*
*   NAME
*   FileInfosC::~FileInfosC -- Destructeur
*
*   SYNOPSIS
*   FileInfosC::~FileInfosC()
*
*   FileInfosC::~FileInfosC();
*
*   FUNCTION
*   Dtruit l'objet FileInfos. Les tampons allous pour la duplication des
*   chanes (nom du fichier et du rpertoire) sont librs.
*
*   SEE ALSO
*   CONSTRUCTOR
*
*****************************************************************************
*
*/


FileInfosC::~FileInfosC()
{
    // Librer les tampons allous pour la duplication du nom du fichier (= nom du noeud) ainsi
    // que de son chemin d'accs. (Cf constructeur de la classe).

    char *s = (char *) NodeName();

    delete [] s;
    delete [] fi_cFullPath;
}
///
/// FileInfosC::ComputeTime()
/****** Class FileInfosC/ComputeTime ****************************************
*
*   NAME
*   FileInfosC::ComputeTime -- Calculer la dure du fichier.
*
*   SYNOPSIS
*   FileInfosC::ComputeTime()
*
*   void FileInfosC::ComputeTime();
*
*   FUNCTION
*   Calcule la dure du fichier en se basant sur le nombre d'chantillons par
*   canal et la frquence d'chantillonage. Si l'une de ces deux valeurs n'
*   est pas connue, la dure du fichier est dfinie avec "88:88".
*
*   SEE ALSO
*   FileInfosC::OverrideSettings()
*
*****************************************************************************
*
*/


void FileInfosC::ComputeTime()
{
    if (fi_iSamplingFreq == 0 || fi_iNumFrames == 0)
    {
        fi_sTime.iTime = 0x5858;     // 88:88
    }
    else
    {
        div_t t;
        ULONG iNumSecs = fi_iNumFrames / fi_iSamplingFreq;

        t = div(iNumSecs, 60);
        fi_sTime.sTime.bMinutes  = t.quot;
        fi_sTime.sTime.bSecondes = t.rem;
    }
}
///
/// FileInfosC::OverrideSettings()
/****** Class FileInfosC/OverrideSettings ***********************************
*
*   NAME
*   FileInfosC::OverrideSettings -- Modifie les rglages.
*
*   SYNOPSIS
*   Result = FileInfosC::OverrideSettings(iSampleFreq, bNumChannels)
*
*   bool FileInfosC::OverrideSettings(ULONG, UBYTE);
*
*   FUNCTION
*   Modifie la frquence d'chantillonage et le nombre de canaux du fichier
*   audio.
*
*   INPUTS
*   iSampleFreq  - Frquence d'chantillonage (Hz)
*   bNumChannels - Nombre de canaux (1 = mono, 2 = stro).
*
*   RESULT
*   Result - TRUE s'il n'y a pas eu d'erreur.
*
*   NOTES
*   La modification de ces valeurs entrane le recalcul de la dure du
*   fichier. D'autre part, les nouvelles valeurs entrainent aussi une
*   modification des rglages de l'encodeur (frquence d'chantillonage et
*   mode d'encodage).
*
*   SEE ALSO
*   FileInfosC::ComputeTime()
*
*****************************************************************************
*
*   15.07.98 : Utilise la frquence d'chantillonage mentionne sur la ligne
*       de commande (ou celle par dfaut) lorsque la fonction reoit la
*       valeur 0 comme frquence. Dans le cas contraire, les fichiers RAW
*       sont toujours encods avec la plus petite frquence possible, c'est
*        dire 32 KHz.
*
*/


int FileInfosC::OverrideSettings(ULONG iSampleFreq, UBYTE bNumChannels)
{
    if (bNumChannels == 0 || bNumChannels > 2) return FALSE;

    if (iSampleFreq == 0)
    {
        iSampleFreq = fi_roDefConfig.GetSampleFreq();
    }

    fi_iSamplingFreq = iSampleFreq;
    fi_bNumChannels  = bNumChannels;
    fi_iTotalSamples = fi_iNumFrames * fi_bNumChannels;

    // Modifier la frquence d'chantillonage utilise par l'encodeur.

    if (!SetSampleFreq(iSampleFreq)) return FALSE;

    // Modifier le mode d'encodage de l'encodeur en fonction du nombre de canaux.
    //
    //  Fichier     Rglage par dfaut      Nouveau rglage
    //  Mono        Dual channel            Dual channel
    //  Mono        #?                      Mono
    //  Stereo      Joint stereo            Joint stereo
    //  Stereo      #?                      Stereo

    e_Encoding eMode = fi_roDefConfig.GetEncodingMode();

    if (bNumChannels == 1)
    {
        SetEncodingMode( (eMode == DUALCHANNEL) ? DUALCHANNEL : MONO);
    }
    else
    {
        SetEncodingMode( (eMode == JOINTSTEREO) ? JOINTSTEREO : STEREO);
    }

    ComputeTime();          // Modifier la dure du fichier.

    return TRUE;
}
///
/// FileInfosC::SetNumFrames()
/****** Class FileInfosC/SetNumFrames ***************************************
*
*   NAME
*   FileInfosC::SetNumFrames -- Dfinir le nombre de frames.
*
*   SYNOPSIS
*   FileInfosC::SetNumFrames(iFrames)
*
*   void FileInfosC::SetNumFrames(ULONG);
*
*   FUNCTION
*   Dfinir le nombre d'chantillons disponibles dans le fichier. Il s'agit
*   du nombre d'chantillons par canal, et non du nombre total d'chantillons
*
*   INPUTS
*   iFrames - Nombre d'chantillons par canal.
*
*   NOTES
*   Cette fonction provoque le recalcul de la dure du fichier audio.
*
*   SEE ALSO
*   FileInfosC::ComputeTime()
*
*****************************************************************************
*
*/


void FileInfosC::SetNumFrames(ULONG iFrames)
{
    fi_iNumFrames    = iFrames;
    fi_iTotalSamples = iFrames * fi_bNumChannels;
    ComputeTime();
}
///
/// FileInfosC::DefineOutputName()
/****** Class FileInfosC/DefineOutputName ***********************************
*
*   NAME
*   FileInfosC::DefineOutputName -- Dfinie le nom du fichier encod.
*
*   SYNOPSIS
*   FileInfosC::DefineOutputName()
*
*   void FileInfosC::DefineOutputName();
*
*   FUNCTION
*   Dtermine le nom du fichier  encoder.
*   La configuration globale de l'encodeur permet trois situations distinctes
*   qui sont :
*   - Aucune destination n'est dfinie. Dans ce cas, le fichier encod est
*     plac dans le mme rpertoire que le fichier audio. L'extension "mp?"
*     est ajoute au nom du fichier aprs suppression de l'extension
*     originale
*   - La destination correspond  un rpertoire. Le fichier encod sera plac
*     dans ce rpertoire, l'extension "mp?" tant ajoute au nom du fichier
*     aprs suppression de l'extension originale.
*   - La destination est un nom de fichier. Le fichier encod portera ce nom
*     et sera plac dans le rpertoire ventuellement indiqu avec ce nom.
*
*   NOTES
*
*   BUGS
*
*****************************************************************************
*
*/


void FileInfosC::DefineOutputName()
{
    char sNameBuff[32];
    char sPathBuff[512];
    char *pos;

    if (GetTargetType() == TARGET_FILE) return;

    if (GetTargetType() == TARGET_DIRECTORY)
    {
        strcpy(sPathBuff, GetOutputName());
    }

    if (GetTargetType() == TARGET_EMPTY)
    {
        strcpy(sPathBuff, GetPathName());
    }

    strncpy(sNameBuff, NodeName(), 26);
    sNameBuff[26] = '\0';

    if ( (pos = strrchr(sNameBuff, '.')) )
        sNameBuff[pos - sNameBuff] = '\0';

    strcat(sNameBuff, asExtensions[GetLayer()]);

    AddPart(sPathBuff, sNameBuff, sizeof(sPathBuff));
    SetOutputName(sPathBuff);
}
///

//----------------------------------------------------------------------------------------------------

/// FileInfosC::GetAudioName()
const char *FileInfosC::GetAudioName() const
{
    switch (NodePri())
    {
        case AFT_AIFF:
            return "IFF-AIFF";

        case AFT_MAUD:
            return "IFF-MAUD";

        case AFT_WAV:
            return "RIFF-WAV";

        case AFT_CDDA:
            return "CD AUDIO Motorola";

        case AFT_CDDA_INTEL:
            return "CD AUDIO Intel";

        case AFT_PCM:
            return "RAW/PCM 16 bits mono";

        case AFT_DEFAULT:
            return GetString(MSG_ASSUME_RAW_TXT);

        case AFT_UNKNOWN:
            return GetString(MSG_UNKNOWN_FILE_FORMAT);
    }
}
///

